searchData={"items":[{"type":"module","title":"Dredd","doc":"Dredd judges data for you. It's a validator for a wide range of\ndatastructures.\n\nSee the [README][1] for a detailed guide.\n\n[1]: readme.html","ref":"Dredd.html"},{"type":"function","title":"Dredd.set_single_error/4","doc":"This is a convenience function in case you want to write your own\nvalidators. It will set the `valid?` flag of the given `Dredd.Dataset` to false.\nIt will also create a `Dredd.SingleError` structure with the given values\nand assing it to the `error` field of the `Dredd.Dataset`.","ref":"Dredd.html#set_single_error/4"},{"type":"function","title":"Dredd.validate_binary/2","doc":"Validates the given values binaries. Optionally also validates the length of the binary either .","ref":"Dredd.html#validate_binary/2"},{"type":"function","title":"Options - Dredd.validate_binary/2","doc":"* `:is` - exact required length of a binary\n* `:min` - minimal required length of a binary\n  (should not be used together with `is`)\n* `:max` - maximal allowed length of a binary\n  (should not be used together with `is`)\n* `:type_message` - error message in case the type is wrong;\n  defaults to \"is not a binary\"\n* `:is_message` - error message in case the exact length is wrong\n   defaults to \"should be %{count} bytes(s)\"\n* `:min_message` - error message in case the length is too short\n  defaults to \"should be at least %{count} bytes(s)\"\n* `:max_message` - error message in case the length is too long\n  defaults to \"should be at \"should be at most %{count} bytes(s)\"","ref":"Dredd.html#validate_binary/2-options"},{"type":"function","title":"Dredd.validate_boolean/2","doc":"Validates the given values is of type boolean. Optionally also validates\nagainst a specific boolean value.","ref":"Dredd.html#validate_boolean/2"},{"type":"function","title":"Options - Dredd.validate_boolean/2","doc":"* `:is` - the expected value (`true`|`false`)\n* `:wrong_type_message` - error message, defaults to \"is not a boolean\"\n* `:wrong_value_message` - error message, defaults to \"expected value: %{expected}\"","ref":"Dredd.html#validate_boolean/2-options"},{"type":"function","title":"Examples - Dredd.validate_boolean/2","doc":"Simple case with data of invalid type:\n```elixir\niex> Dredd.validate_boolean('foo')\n%Dredd.Dataset{\n  data: 'foo',\n  error: %Dredd.SingleError{\n    validator: :boolean,\n    message: \"is not a boolean\",\n    metadata: %{kind: :type}\n  },\n  valid?: false\n}\n```\n\nSimple case with valid boolean type:\n```elixir\niex> Dredd.validate_boolean(true)\n%Dredd.Dataset{data: true, error: nil, valid?: true}\n```\n\nSpecial invalid case with optional expected value:\n```elixir\niex> Dredd.validate_boolean(false, is: true)\n%Dredd.Dataset{\n  data: false,\n  error: %Dredd.SingleError{\n    validator: :boolean,\n    message: \"expected value: %{expected}\",\n    metadata: %{expected: true, kind: :value}\n  },\n  valid?: false\n}\n```","ref":"Dredd.html#validate_boolean/2-examples"},{"type":"function","title":"Dredd.validate_email/2","doc":"Validates if the value of a given field is an email.\n\nNOTE: this validator is not RFC822 compliant. If you really need to be sure,\nsend an email to that address.","ref":"Dredd.html#validate_email/2"},{"type":"function","title":"Options - Dredd.validate_email/2","doc":"* `:message` - error message, default to \"is not a valid email address\"","ref":"Dredd.html#validate_email/2-options"},{"type":"function","title":"Dredd.validate_exclusion/3","doc":"Validates the value for the given field is not contained within the provided\nenumerable.","ref":"Dredd.html#validate_exclusion/3"},{"type":"function","title":"Options - Dredd.validate_exclusion/3","doc":"* `:message` - error message, defaults to \"is reserved\"","ref":"Dredd.html#validate_exclusion/3-options"},{"type":"function","title":"Dredd.validate_format/3","doc":"Validates the value of the given field matches the provided format.","ref":"Dredd.html#validate_format/3"},{"type":"function","title":"Options - Dredd.validate_format/3","doc":"* `:message` - error message, defaults to \"has invalid format\"","ref":"Dredd.html#validate_format/3-options"},{"type":"function","title":"Dredd.validate_inclusion/3","doc":"Validates the value for the given field is contained within the provided\nenumerable.","ref":"Dredd.html#validate_inclusion/3"},{"type":"function","title":"Options - Dredd.validate_inclusion/3","doc":"* `:message` - error message, defaults to \"is invalid\"","ref":"Dredd.html#validate_inclusion/3-options"},{"type":"function","title":"Dredd.validate_list/3","doc":"Applies a validator function to a each element of a list contained in a field.","ref":"Dredd.html#validate_list/3"},{"type":"function","title":"Options - Dredd.validate_list/3","doc":"* `:is` - exact required length of a list\n* `:min` - minimal required length of a list \n  (should not be used together with `is`)\n* `:max` - maximal allowed length of a list\n  (should not be used together with `is`)\n* `:type_message` - error message in case the type is wrong;\n  defaults to \"is not a list\"\n* `:is_message` - error message in case the exact length is wrong\n   defaults to \"should be %{count} item(s)\"\n* `:min_message` - error message in case the length is too short\n  defaults to \"should be at least %{count} item(s)\"\n* `:max_message` - error message in case the length is too long\n  defaults to \"should be at \"should be at most %{count} item(s)\"","ref":"Dredd.html#validate_list/3-options"},{"type":"function","title":"Example - Dredd.validate_list/3","doc":"```elixir\n```","ref":"Dredd.html#validate_list/3-example"},{"type":"function","title":"Dredd.validate_map/3","doc":"Validates the structure of a Map, Keyword List, Struct or anything else\nthat supports the Access behaviour and whose structure can be represented\nby a map.","ref":"Dredd.html#validate_map/3"},{"type":"function","title":"Options - Dredd.validate_map/3","doc":"* `message` - error message in case the type-check fails\n   defaults to: \"is not a map\"","ref":"Dredd.html#validate_map/3-options"},{"type":"function","title":"Example - Dredd.validate_map/3","doc":"```elixir\niex> value = %{ field_a: 10, field_b: \"foo\" }\n%{field_a: 10, field_b: \"foo\"}\niex> validator_map = %{\n...>   field_a: &Dredd.validate_string/1,\n...>   field_b: fn data -> Dredd.validate_number(data, :integer) end\n...>}\niex> Dredd.validate_map(value, validator_map)\n%Dredd.Dataset{\n  data: %{field_a: 10, field_b: \"foo\"},\n  error: %Dredd.MapErrors{\n    validator: :map,\n    errors: %{\n      field_a: %Dredd.SingleError{\n        validator: :string,\n        message: \"is not a string\",\n        metadata: %{kind: :type}\n      },\n      field_b: %Dredd.SingleError{\n        validator: :number,\n        message: \"has incorrect numerical type\",\n        metadata: %{kind: :integer}\n      }\n    }\n  },\n  valid?: false\n}\n```","ref":"Dredd.html#validate_map/3-example"},{"type":"function","title":"Dredd.validate_nanoid/2","doc":"Validates if the value of a given field is a UUID.binary_to_string!","ref":"Dredd.html#validate_nanoid/2"},{"type":"function","title":"Options - Dredd.validate_nanoid/2","doc":"* `:message` - error message, defaults to \"is not a valid UUID\"","ref":"Dredd.html#validate_nanoid/2-options"},{"type":"function","title":"Dredd.validate_number/3","doc":"Validates that the value of a field is a number.\n\nSupported types:\n\n* `:float`\n* `:integer`\n* `:non_neg_integer`\n* `:pos_integer`","ref":"Dredd.html#validate_number/3"},{"type":"function","title":"Options - Dredd.validate_number/3","doc":"* ´predicate` - function of type `(any -> boolean)`\n* `:type_message` - error message in case the type is wrong.\n  Defaults to \"has incorrect numerical type\"\n* `:predicate_message` - error message in case the predicate check fails.\n  Defaults to \"violates the given predicate\"","ref":"Dredd.html#validate_number/3-options"},{"type":"function","title":"Dredd.validate_string/2","doc":"Validates the given values is of type string. Optionally also validates\nthe length of the string either as codepoints or graphemes.","ref":"Dredd.html#validate_string/2"},{"type":"function","title":"Options - Dredd.validate_string/2","doc":"* `:is` - exact required length of a string\n* `:min` - minimal required length of a string\n  (should not be used together with `is`)\n* `:max` - maximal allowed length of a string\n  (should not be used together with `is`)\n* `:count` - can be `:codepoints` or `:graphemes`.\n   Defaults to `:graphemes`\n* `:type_message` - error message in case the type is wrong; \n  defaults to \"is not a string\"\n* `:is_message` - error message in case the exact length is wrong\n   defaults to \"should be %{count} character(s)\"\n* `:min_message` - error message in case the length is too short \n  defaults to \"should be at least %{count} character(s)\"\n* `:max_message` - error message in case the length is too long\n  defaults to \"should be at \"should be at most %{count} character(s)\"","ref":"Dredd.html#validate_string/2-options"},{"type":"function","title":"Examples - Dredd.validate_string/2","doc":"Here's a simple data with invalid data:\n```elixir\niex> Dredd.validate_string(10)\n%Dredd.Dataset{\n  data: 10,\n  error: %Dredd.SingleError{\n    validator: :string,\n    message: \"is not a string\",\n    metadata: %{kind: :type}\n  },\n  valid?: false\n}\n```\n\nHere's an example with a string that is too short but has a length\nrequirement:\n```elixir\niex> Dredd.validate_string(\"\", min_length: 5)\n%Dredd.Dataset{\n  data: \"\",\n  error: %Dredd.SingleError{\n    validator: :string,\n    message: \"should be at least %{count} character(s)\",\n    metadata: %{count: 5, kind: :min_length}\n  },\n  valid?: false\n}\n```","ref":"Dredd.html#validate_string/2-examples"},{"type":"function","title":"Dredd.validate_uuid/2","doc":"Validates if the value of a given field is a UUID.binary_to_string!","ref":"Dredd.html#validate_uuid/2"},{"type":"function","title":"Options - Dredd.validate_uuid/2","doc":"* `:message` - error message, defaults to \"is not a valid UUID\"","ref":"Dredd.html#validate_uuid/2-options"},{"type":"type","title":"Dredd.field_spec/0","doc":"","ref":"Dredd.html#t:field_spec/0"},{"type":"type","title":"Dredd.number_t/0","doc":"","ref":"Dredd.html#t:number_t/0"},{"type":"type","title":"Dredd.single_validator_fun/0","doc":"","ref":"Dredd.html#t:single_validator_fun/0"},{"type":"type","title":"Dredd.validator_map/0","doc":"","ref":"Dredd.html#t:validator_map/0"},{"type":"module","title":"Dredd.Dataset","doc":"This is the internal datastructure used for passing the current validation\nstate from validator to validator including the top-level error-structure of any\nvalidated datastructure.","ref":"Dredd.Dataset.html"},{"type":"function","title":"Dredd.Dataset.new/1","doc":"","ref":"Dredd.Dataset.html#new/1"},{"type":"type","title":"Dredd.Dataset.error_t/0","doc":"","ref":"Dredd.Dataset.html#t:error_t/0"},{"type":"type","title":"Dredd.Dataset.t/0","doc":"","ref":"Dredd.Dataset.html#t:t/0"},{"type":"module","title":"Dredd.ListErrors","doc":"This datastructure shows where in a list validations failed.","ref":"Dredd.ListErrors.html"},{"type":"type","title":"Dredd.ListErrors.error_t/0","doc":"","ref":"Dredd.ListErrors.html#t:error_t/0"},{"type":"type","title":"Dredd.ListErrors.t/0","doc":"","ref":"Dredd.ListErrors.html#t:t/0"},{"type":"module","title":"Dredd.MapErrors","doc":"This datastructure shows for which fields in a struct validation failed.","ref":"Dredd.MapErrors.html"},{"type":"type","title":"Dredd.MapErrors.error_t/0","doc":"","ref":"Dredd.MapErrors.html#t:error_t/0"},{"type":"type","title":"Dredd.MapErrors.t/0","doc":"","ref":"Dredd.MapErrors.html#t:t/0"},{"type":"module","title":"Dredd.SingleError","doc":"Represents an error on a single Value.","ref":"Dredd.SingleError.html"},{"type":"type","title":"Dredd.SingleError.t/0","doc":"","ref":"Dredd.SingleError.html#t:t/0"},{"type":"extras","title":"Dredd","doc":"# Dredd\nDredd judges your data. It can validate arbitrary Elixir data with arbitrary deep nesting.\n\n\n[![Build Status](https://github.com/prakti/dredd/workflows/ci/badge.svg)](https://github.com/prakti/dredd/actions) [![Hex.pm Version](https://img.shields.io/hexpm/v/dredd)](https://hex.pm/packages/dredd) [![Static Badge](https://img.shields.io/badge/hex-docs-green)](https://hexdocs.pm/dredd/readme.html)\n\nStarted as a fork of [Justify][2], dredd is a total rewrite to handle plain values\nas well as deeply nested structures and return parseable error-structures.\n\nSimilar to [Ecto.Changeset][1], Dredd does not use a schema and a lot of metaprogramming magic.\nEverything is done by composing or piping functions. Dredd excels in use-cases, where you want\nto either avoid Ecto as a dependency, and/or where you want to validate deeply nested datastructures.\n\nDredd boasts an extensive test-suite, using property-based tests to verify its capabilities across\nall possible input data. There should only be a very small amount of unspecified behaviour in Dredd.\n\n[1]: https://hexdocs.pm/ecto/Ecto.Changeset.html\n[2]: https://github.com/malomohq/justify\n\nPlease see the [Changelog](changelog.html) for what has changed in the latest release.","ref":"readme.html"},{"type":"extras","title":"A Basic Example - Dredd","doc":"```elixir\niex> Dredd.validate_number(\"this is not a number\", :integer)\n%Dredd.Dataset{\n  data: \"this is not a number\",\n  error: %Dredd.SingleError{\n    validator: :number,\n    message: \"has incorrect numerical type\",\n    metadata: %{kind: :integer}\n  },\n  valid?: false\n}\n```\n\nAs you can see this is a rather verbose output; but we believe that this can\ncome in handy in conjunction when you have to translate the error on the\nuser interface.\n\nEach validation function will return a `Dredd.Dataset` which can be\npassed into the next function. If a validation error is encountered the dataset\nwill be marked as invalid and an error will be added to the struct. Each\nvalidator behaves in a fail-fast manner. If the Dataset is already invalid\nno further validation happens for any given value. As a tip: you should order your\nvalidations by descending importance.\n\nErrors are distinguished by type to allow traversal of nested structures by\nmatching against the struct type of the encountered error.\n\nWhen validating single values a `Dredd.SingleError` is returned whenever it\nfails.","ref":"readme.html#a-basic-example"},{"type":"extras","title":"Validating Lists - Dredd","doc":"To ramp up the complexity: you can validate all elements of a list by handing\na validation function to `Dredd.validate_list/3`.","ref":"readme.html#validating-lists"},{"type":"extras","title":"Simple List Example - Dredd","doc":"This is how you would validate a list of strings.\n\n```elixir\niex(2)> Dredd.validate_list([\"string\", -1, \"string\", 0], &Dredd.validate_string/1)\n%Dredd.Dataset{\n  data: [\"string\", -1, \"string\", 0],\n  error: %Dredd.ListErrors{\n    validator: :list,\n    errors: %{\n      1 => %Dredd.SingleError{\n        validator: :string,\n        message: \"is not a string\",\n        metadata: %{kind: :type}\n      },\n      3 => %Dredd.SingleError{\n        validator: :string,\n        message: \"is not a string\",\n        metadata: %{kind: :type}\n      }\n    }\n  },\n  valid?: false\n}\n```\n\nIf the list does not match given length requirements or the given data is not a list, the\nreturned `Dredd.Dataset` will be invalid and contain a `Dredd.SingleError`.\n\nIn case an element of the list is invalid, the returned `Dredd.Dataset` will\ncontain `Dredd.ListErrors` with its `errors` field containing a map. The keys\nof that map are the indices of the values for which validation failed. The\nvalues of that map are `Dredd.SingleError`, `Dredd.ListErrors` or\n`Dredd.MapErrors` depending on the given validator for the list items.\n\nThis distinction is meant to help with parsing the errors of nested validations.","ref":"readme.html#simple-list-example"},{"type":"extras","title":"Complex List Example - Dredd","doc":"You can exploit the fact that you can pipe the output of all validator\nfunctions into the next one and compose chain together multiple values for\nthe elements of your list.\n\n```elixir\niex> item_validator = fn data ->\n...>   Dredd.validate_string(data)\n...>   |> Dredd.validate_email()\n...> end\niex> Dredd.validate_list([\"foo\", \"foo@bar.com\"], item_validator)\n%Dredd.Dataset{\n  data: [\"foo\", \"foo@bar.com\"],\n  error: %Dredd.ListErrors{\n    validator: :list,\n    errors: %{\n      0 => %Dredd.SingleError{\n        validator: :email,\n        message: \"is not a valid email address\",\n        metadata: %{}\n      }\n    }\n  },\n  valid?: false\n}\n```","ref":"readme.html#complex-list-example"},{"type":"extras","title":"Handling Non-Lists - Dredd","doc":"In case `Dredd.validate_list/3` is handed anything but a list, it will return\na `Dredd.SingleError` indicating that the value is of wrong type:\n\n```elixir\niex> Dredd.validate_list(100, &Dredd.validate_email/1)\n%Dredd.Dataset{\n  data: 100,\n  error: %Dredd.SingleError{\n    validator: :list,\n    message: \"is not a list\",\n    metadata: %{kind: :type}\n  },\n  valid?: false\n}\n```","ref":"readme.html#handling-non-lists"},{"type":"extras","title":"Validating Structs - Dredd","doc":"You can validate fields of a struct or map using the `Dredd.validate_map/2` function.","ref":"readme.html#validating-structs"},{"type":"extras","title":"Simple Struct Example - Dredd","doc":"```elixir\niex> value = %{ field_a: 10, field_b: \"foo\" }\n%{field_a: 10, field_b: \"foo\"}\niex> validator_map = %{\n...>   field_a: &Dredd.validate_string/1,\n...>   field_b: fn data -> Dredd.validate_number(data, :integer) end\n...>}\niex> Dredd.validate_map(value, validator_map)\n%Dredd.Dataset{\n  data: %{field_a: 10, field_b: \"foo\"},\n  error: %Dredd.MapErrors{\n    validator: :map,\n    errors: %{\n      field_a: %Dredd.SingleError{\n        validator: :string,\n        message: \"is not a string\",\n        metadata: %{kind: :type}\n      },\n      field_b: %Dredd.SingleError{\n        validator: :number,\n        message: \"has incorrect numerical type\",\n        metadata: %{kind: :integer}\n      }\n    }\n  },\n  valid?: false\n}\n```\n\nIn case of errors the output of `Dredd.validate_map\\3` is quite similar to that\nof `Dredd.validate_list\\3`. \n\nIf the give value is not a valid map the `Dredd.Dataset` will contain a\n`Dredd.SingleError`. \n\n\nIf the validations on field-level failed the `Dredd.Dataset` will contain `Dredd.MapErrors`.\nThe keys in that map are the fieldnames of the invalid fields. The values in the map can be\n`Dredd.SingleError`, `Dredd.ListErrors` or `Dredd.MapErrors` depending on the\nvalidator of that field.\n\nThis distinction is meant to help with parsing the errors of nested validations.","ref":"readme.html#simple-struct-example"},{"type":"extras","title":"Nested Example - Dredd","doc":"```elixir\niex> email_list_validator = fn data ->\n...>   Dredd.validate_list(data, &Dredd.validate_email/1)\n...> end\n# Function<...>\niex> validate_embedded_map = fn data ->\n...>   validator_map = %{\n...>     email_list: email_list_validator,\n...>     number: &Dredd.validate_number(&1, :integer)\n...>   }\n...>   Dredd.validate_map(data, validator_map)\n...> end\n# Function<...>\niex> map_list_validator = fn data ->\n...>   Dredd.validate_list(data, validate_embedded_map)\n...> end\n# Function<...>\niex> validator_map = %{\n...>   map_list: map_list_validator,\n...>   str_field: &Dredd.validate_string/1\n...> }\n%{\n  map_list: #Function<...>,\n  str_field: &Dredd.validate_string/1\n}\niex> value = %{\n...>   map_list: [\n...>    %{email_list: [\"foo@bar.com\", \"bang@baz.net\"], number: 10 },\n...>    %{email_list: [\"foo@bar.com\", \"blubb\"], number: 20 }\n...>   ],\n...>   str_field: \"bar\"\n...> }\n%{\n  map_list: [\n    %{email_list: [\"foo@bar.com\", \"bang@baz.net\"], number: 10},\n    %{email_list: [\"foo@bar.com\", \"blubb\"], number: 20}\n  ],\n  str_field: \"bar\"\n}\niex> Dredd.validate_map(value, validator_map)\n%Dredd.Dataset{\n  data: %{\n    map_list: [\n      %{email_list: [\"foo@bar.com\", \"bang@baz.net\"], number: 10},\n      %{email_list: [\"foo@bar.com\", \"blubb\"], number: 20}\n    ],\n    str_field: \"bar\"\n  },\n  error: %Dredd.MapErrors{\n    validator: :map,\n    errors: %{\n      map_list: %Dredd.ListErrors{\n        validator: :list,\n        errors: %{\n          1 => %Dredd.MapErrors{\n            validator: :map,\n            errors: %{\n              email_list: %Dredd.ListErrors{\n                validator: :list,\n                errors: %{\n                  1 => %Dredd.SingleError{\n                    validator: :email,\n                    message: \"is not a valid email address\",\n                    metadata: %{}\n                  }\n                }\n              }\n            }\n          }\n        }\n      }\n    }\n  },\n  valid?: false\n}\n```\n\nIn the example above, the second email in the list of the second sub-map in the\n`map_list` field is invalid. Even with all the extra information the output is\nquite confusing. That's the primary reason we've added all the extra type information.\nWith that you can pattern match against the `struct` type and have a fighting chance\nat successfully traversing the data.","ref":"readme.html#nested-example"},{"type":"extras","title":"Copyright and License - Dredd","doc":"Copyright (c) 2022 Marcus Autenrieth\n\nDredd is licensed under the MIT License, see LICENSE.md for details.","ref":"readme.html#copyright-and-license"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\nAll notable changes to [Dredd](https://github.com/Prakti/dredd) will be documented in this file.\n\nThe format is based on [Keep a Changelog](https://keepachangelog.com/en/1.0.0/),\nand this project adheres to [Semantic Versioning](https://semver.org/spec/v2.0.0.html).\n\n## [2.0.0] - 2022-12-28","ref":"changelog.html"},{"type":"extras","title":"Breaking Changes - Changelog","doc":"This is a complete rewrite from the structure inherited by Justify. There are two reasons for the overhaul:\n- I have a need for more deeply nested datastructures and needed the more verbose errors to keep output manageable.\n- I wanted to improve the composability of validators. As it were, the architecture looked a bit lopsided.","ref":"changelog.html#breaking-changes"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,50,46,49]}}